@@ -12,7 +12,7 @@ from keyword import iskeyword
 import attr
 import pycodestyle
 
-__version__ = "22.4.25"
+__version__ = "22.7.1"
 
 LOG = logging.getLogger("flake8.bugbear")
 CONTEXTFUL_NODES = (
@@ -26,6 +26,7 @@ CONTEXTFUL_NODES = (
     ast.DictComp,
     ast.GeneratorExp,
 )
+FUNCTION_NODES = (ast.AsyncFunctionDef, ast.FunctionDef, ast.Lambda)
 
 Context = namedtuple("Context", ["node", "stack"])
 
@@ -155,12 +156,13 @@ class BugBearChecker:
             return True
 
         for i in range(2, len(code) + 1):
-            if code[:i] in self.options.select:
+            if self.options.select and code[:i] in self.options.select:
                 return True
 
             # flake8 >=4.0: Also check for codes in extend_select
             if (
                 hasattr(self.options, "extend_select")
+                and self.options.extend_select
                 and code[:i] in self.options.extend_select
             ):
                 return True
@@ -197,6 +199,23 @@ def _to_name_str(node):
         return _to_name_str(node.value)
 
 
+def names_from_assignments(assign_target):
+    if isinstance(assign_target, ast.Name):
+        yield assign_target.id
+    elif isinstance(assign_target, ast.Starred):
+        yield from names_from_assignments(assign_target.value)
+    elif isinstance(assign_target, (ast.List, ast.Tuple)):
+        for child in assign_target.elts:
+            yield from names_from_assignments(child)
+
+
+def children_in_scope(node):
+    yield node
+    if not isinstance(node, FUNCTION_NODES):
+        for child in ast.iter_child_nodes(node):
+            yield from children_in_scope(child)
+
+
 def _typesafe_issubclass(cls, class_or_tuple):
     try:
         return issubclass(cls, class_or_tuple)
@@ -219,6 +238,7 @@ class BugBearVisitor(ast.NodeVisitor):
     contexts = attr.ib(default=attr.Factory(list))
 
     NODE_WINDOW_SIZE = 4
+    _b023_seen = attr.ib(factory=set, init=False)
 
     if False:
         # Useful for tracing what the hell is going on.
@@ -347,6 +367,31 @@ class BugBearVisitor(ast.NodeVisitor):
     def visit_For(self, node):
         self.check_for_b007(node)
         self.check_for_b020(node)
+        self.check_for_b023(node)
+        self.generic_visit(node)
+
+    def visit_AsyncFor(self, node):
+        self.check_for_b023(node)
+        self.generic_visit(node)
+
+    def visit_While(self, node):
+        self.check_for_b023(node)
+        self.generic_visit(node)
+
+    def visit_ListComp(self, node):
+        self.check_for_b023(node)
+        self.generic_visit(node)
+
+    def visit_SetComp(self, node):
+        self.check_for_b023(node)
+        self.generic_visit(node)
+
+    def visit_DictComp(self, node):
+        self.check_for_b023(node)
+        self.generic_visit(node)
+
+    def visit_GeneratorExp(self, node):
+        self.check_for_b023(node)
         self.generic_visit(node)
 
     def visit_Assert(self, node):
@@ -519,6 +564,59 @@ class BugBearVisitor(ast.NodeVisitor):
                 n = targets.names[name][0]
                 self.errors.append(B020(n.lineno, n.col_offset, vars=(name,)))
 
+    def check_for_b023(self, loop_node):
+        """Check that functions (including lambdas) do not use loop variables.
+
+        https://docs.python-guide.org/writing/gotchas/#late-binding-closures from
+        functions - usually but not always lambdas - defined inside a loop are a
+        classic source of bugs.
+
+        For each use of a variable inside a function defined inside a loop, we
+        emit a warning if that variable is reassigned on each loop iteration
+        (outside the function).  This includes but is not limited to explicit
+        loop variables like the `x` in `for x in range(3):`.
+        """
+        # Because most loops don't contain functions, it's most efficient to
+        # implement this "backwards": first we find all the candidate variable
+        # uses, and then if there are any we check for assignment of those names
+        # inside the loop body.
+        suspicious_variables = []
+        for node in ast.walk(loop_node):
+            if isinstance(node, FUNCTION_NODES):
+                argnames = {
+                    arg.arg for arg in ast.walk(node.args) if isinstance(arg, ast.arg)
+                }
+                if isinstance(node, ast.Lambda):
+                    body_nodes = ast.walk(node.body)
+                else:
+                    body_nodes = itertools.chain.from_iterable(map(ast.walk, node.body))
+                for name in body_nodes:
+                    if (
+                        isinstance(name, ast.Name)
+                        and name.id not in argnames
+                        and isinstance(name.ctx, ast.Load)
+                    ):
+                        err = B023(name.lineno, name.col_offset, vars=(name.id,))
+                        if err not in self._b023_seen:
+                            self._b023_seen.add(err)  # dedupe across nested loops
+                            suspicious_variables.append(err)
+
+        if suspicious_variables:
+            reassigned_in_loop = set(self._get_assigned_names(loop_node))
+
+        for err in sorted(suspicious_variables):
+            if reassigned_in_loop.issuperset(err.vars):
+                self.errors.append(err)
+
+    def _get_assigned_names(self, loop_node):
+        loop_targets = (ast.For, ast.AsyncFor, ast.comprehension)
+        for node in children_in_scope(loop_node):
+            if isinstance(node, (ast.Assign)):
+                for child in node.targets:
+                    yield from names_from_assignments(child)
+            if isinstance(node, loop_targets + (ast.AnnAssign, ast.AugAssign)):
+                yield from names_from_assignments(node.target)
+
     def check_for_b904(self, node):
         """Checks `raise` without `from` inside an `except` clause.
 
@@ -824,6 +922,11 @@ class B020NameFinder(NameFinder):
     def visit_comprehension(self, node):
         self.visit(node.iter)
 
+    def visit_Lambda(self, node):
+        self.visit(node.body)
+        for lambda_arg in node.args.args:
+            self.names.pop(lambda_arg.arg, None)
+
 
 error = namedtuple("error", "lineno col message type vars")
 Error = partial(partial, error, type=BugBearChecker, vars=())
@@ -1023,18 +1126,20 @@ B020 = Error(
 )
 B021 = Error(
     message=(
-        "B021 f-string used as docstring."
+        "B021 f-string used as docstring. "
         "This will be interpreted by python as a joined string rather than a docstring."
     )
 )
 B022 = Error(
     message=(
-        "B022 No arguments passed to `contextlib.suppress`."
-        "No exceptions will be suppressed and therefore this"
+        "B022 No arguments passed to `contextlib.suppress`. "
+        "No exceptions will be suppressed and therefore this "
         "context manager is redundant."
     )
 )
 
+B023 = Error(message="B023 Function definition does not bind loop variable {!r}.")
+
 # Warnings disabled by default.
 B901 = Error(
     message=(
